"use strict";

const Logger 		= require("berry-frame/utils/Logger.js").Logger;
const isMissing 	= require("berry-frame/utils/Logger.js").isMissing;
const isPresent 	= require("berry-frame/utils/Logger.js").isPresent;
const App 			= require("berry-frame/server/App.js").App;
const theServer		= require("berry-frame/server/Server.js").theServer;
const theHardware	= require("berry-frame/hw_control/Hardware.js").theHardware;
const fs			= require("fs");
const jpeg 			= require('jpeg-js');
const UTIF 			= require('utif');
const spawnProcess	= require('child_process').spawn;
const { Image } 	= require('image-js');

var app;
const IMAGE_DIR		= "/mnt/RAMDisk/camera";

// =========================================================================================================

class EasyDeal extends App {
	/*
		EasyDeal controls a Bridge Card Dealing Enine.
		
		Right now it reads a file with a card image and identifies the card.
	*/
	
	constructor(hardware,silent) {
		super(hardware,silent);
		
		// make instance available for functions which are invoked as callbacks
		app=this;
	}

	log(msg) {
		Logger.log(Date.now()%100000+"\tEasyDeal.... "+msg);
	}
	
	init() {
		this.ranks=["A","4","J","5","3","9","6","K","8","Q","2","Z","7"];
		this.cardNr = -1;
		this.scanner = new CardScanner("EasyDeal/server/ranks.json");
		this.engine = new EngineInterface();
		this.engine.start("/dev/ttyACM0",115200);
		var elm = app.hardware.elms["cameraSettings"];
		this.setCamera(elm,elm.dev.getValue(),null);
		this.pbns=[];
	}
	
	onStart(onStarted) {

		// create list of available PBN libraries, open first deal of first lib
		this.findPBNs("");
		// update option list on client and expand combo box
		theHardware.elms['pbns'].options=this.pbns;
		theServer.broadcastState({id:"pbns"},"Action",{options:this.pbns,size:10});
		this.selectPBN(null,this.pbns[0]);

		// create and start background process for loading images from camera stream
		this.imageLoader = new ImageLoader(110,290,130,100);
		this.imageLoader.start(-1);	// collect frames forever

		// trigger an initial card scan after some time (which is needed to start the loader)
		setTimeout( function() { app.scan(); } ,2000 );
	}
	
	onStop(onFinished) {
	}

	onClientConnect(socket,hostname) {
		// called after the client has received its full setup
		var path= (theServer.hostname=="localhost") ? 
			"http://localhost/bridge/EasyDeal/html/games/ifm_local.php" : 
			"http://"+theServer.hostname+"/games/ifm.php"
		;
		app.hardware.elms["fileMgr"].name="<a title='PBN Dateine hochladen und verwalten' href='"+path+"'>PBN ..</a>";
		theServer.sendResponse(socket,app.hardware.elms.fileMgr);
	}
	
	setCamera(elm,value,action) {
		var arg = value;
		elm.dev.setValue(value);
		if (typeof arg == "string") arg=JSON.parse(("{"+arg.replace(/=/g,':')+"}").replace(/([a-zA-Z_0-9]+) ?:/g,'"$1":'));
		if (typeof arg.bright) {
			if (require('os').platform() != "win32") {
				const execSync = require('child_process').execSync;
				this.log("setting camera brightness to "+arg.bright);
				execSync("v4l2-ctl --set-ctrl=brightness="+arg.bright);
			}
		}		
		if (typeof arg.focus) {
			if (require('os').platform() != "win32") {
				const execSync = require('child_process').execSync;
				this.log("setting camera focus to "+arg.focus);
				execSync("v4l2-ctl --set-ctrl=focus_auto=0");
				execSync("v4l2-ctl --set-ctrl=focus_absolute="+arg.focus);
			}
		}	
		if (typeof arg.x !="undefined") {
			this.log("setting camera crop area to "+arg.w+"x"+arg.h+" at "+arg.x+"/"+arg.y);
			this.setCropRegion({x:arg.x,y:arg.y,width:arg.w,height:arg.h});
		}		
	}
	
	selectPBN(onDone,file) {
		this.pbnLib = new PbnLib("../html/games/"+file);
		
		// update option list and shrink combo box
		theServer.broadcastState({id:"pbns"},"Action",{size:1});
		theHardware.elms['deals'].options=this.pbnLib.dealIds;
		theServer.broadcastState({id:"deals"},"Action",{options:this.pbnLib.dealIds});
		this.selectDeal(null,"1");			
	}
	
	findPBNs(relPath) {
		var path = relPath=="" ? "../html/games" : "../html/games/"+relPath;
		this.log("searching PBN at "+path);
		if (fs.existsSync(path) && fs.lstatSync(path).isDirectory()) {
			fs.readdirSync(path).forEach(member => {
				if (fs.lstatSync(path+"/"+member).isDirectory()) {
					this.log("found subdirectory "+member);
					if (member!="." && member!="..") this.findPBNs(relPath=="" ? member : relPath+"/"+member);
				}
				else if (member.substr(-4).toUpperCase()==".PBN") {
					this.log("found PBN "+path+"/"+member);
					this.pbns.push(relPath+"/"+member);
				}
			});
		}
		this.dealNumber=1;
	}
	
	selectDeal(onDone,dealId) {
		this.dealNumber=parseInt(dealId);
		app.pbnDeal=app.pbnLib.deals[this.dealNumber-1];
		app.hardware.elms["hands"].dev.clear();
		app.hardware.elms["hands"].dev.setValue(app.pbnDeal.layout);
	}
	
	setCropRegion(region) {
		this.scanner.setCropRegion(region);	
	}

	setRotation(rot) {
		this.scanner.setRotation(rot);	
	}
	
	zeroPos() {
		this.engine.zeroPos();
	}
	
	scan() {
		this.scanner.scanOnly();
	}

	deal1() {
		// scan and deal the bottom card
		this.scanner.scanAndDeal(detected => {
			// if successful: scan the next card
			if (detected) this.scanner.scanOnly();
		});
	}

	nextDeck() {
		if (app.pbnLib.deals.length>this.dealNumber) {
			this.log("loading next deck after "+this.dealNumber);
			this.selectDeal(null,""+(this.dealNumber+1));
		}
		else {
			this.log("restarting with first deck after "+this.dealNumber);
			this.selectDeal(null,"1");	// wrap around
		}
		theServer.broadcastState(
			{id:"deals"},
			"Action",
			{options:this.pbnLib.dealIds,selected:this.pbnLib.dealIds[this.dealNumber-1]}
		);
	}
	
	async dealDeck() {
		await this.engine.zeroPos();
		await this.engine.withDraw();
		let whenDealt= async detected => {
			if (!detected) {
				app.log("dealDeck: ERROR at card #"+(app.cardNr));
				return false;
			}
			if (app.cardNr==52) {
				await app.engine.unloadPos();
				app.nextDeck();
				app.scan();
				app.log("dealDeck: done");
				return true;
			}
			else {
				app.scanner.scanAndDeal(whenDealt);
			}
		}
		this.cardNr=0;
		whenDealt(true);
	}
	
	nextRankSymbol(symbol) {
		var r=symbol.substring(0,1);
		var t=symbol.substring(1);
		if (r=="2") return "3"+t;
		if (r=="3") return "4"+t;
		if (r=="4") return "5"+t;
		if (r=="5") return "6"+t;
		if (r=="6") return "7"+t;
		if (r=="7") return "8"+t;
		if (r=="8") return "9"+t;
		if (r=="9") return "T"+t;
		if (r=="T") return "J"+t;
		if (r=="J") return "Q"+t;
		if (r=="Q") return "K"+t;
		if (r=="K") return "A"+t;
		if (r=="A") return "2"+t;
	}

	async learnRank(elm,rank,action) {
		this.scanner.scan(true,rank,0,detected => {
			fs.writeFileSync(
				"EasyDeal/server/ranks.json",
				JSON.stringify(app.scanner.rankFeatures)
			);
			app.scan();	// scan the next card
		});
	}

	ejectNorth(onDone) 	{ this.engine.ejectBin("N"); }
	ejectEast(onDone)  	{ this.engine.ejectBin("E"); }
	ejectSouth(onDone) 	{ this.engine.ejectBin("S"); }
	ejectWest(onDone)  	{ this.engine.ejectBin("W"); }
	eject(onDone) 		{ this.engine.ejectBin("."); }
}
EasyDeal.getApiDescription = function() {
	return [
		{	cmd:	"setCamera",
			args: [
				{ name : "setting",	meaning: "a combination of name=value, e.g. brightness=120" },
			],
			effect:	"applies the setting to the camera",
		},
		{	cmd:	"setCropRegion",
			args: [
				{ name : "region",	meaning: "x=.., y=.., width=.., height=.." },
			],
			effect:	"defines the region to be cropped",
		},
		{	cmd:	"learnRank",
			args: [
				{ name : "symbol",	meaning: "the rank letter, followed by information about font or catd deck style" },
			],
			effect:	"defines the region to be cropped",
		},
		{	cmd:	"zeroPos",
			effect:	"find zero position for bin rotor",
		},
		{	cmd:	"scan",
			effect:	"scan a card image file and return the ID in the 'Display' with id 'card'",
		},
		{	cmd:	"deal1",
			effect:	"scan a card image and deal the card to the appropriate player",
		},
		{	cmd:	"dealDeck",
			effect:	"scan and deal one deck of 52 cards",
		},
		{	cmd:	"ejectNorth",
			effect:	"drop the bottom card into the North bin",
		},
		{	cmd:	"ejectEast",
			effect:	"drop the bottom card into the East bin",
		},
		{	cmd:	"ejectSouth",
			effect:	"drop the bottom card into the South bin",
		},
		{	cmd:	"ejectWest",
			effect:	"drop the bottom card into the West bin",
		},
		{	cmd:	"eject",
			args: [
				{ name : "bin",	meaning: "1..4 = bin number" },
			],
			effect:	"eject the bottom card and guide it to the given bin",
		},
		{	cmd:	"selectPBN",
			effect:	"select a PBN file, path is relative to 'games' directory",
		},
		{	cmd:	"selectDeal",
			effect:	"select a deal from the current PBN file",
		},
	];
}

// =========================================================================================================
module.exports.EasyDeal = EasyDeal;

var resolveAfter = function(delay) {
	return new Promise(resolve => {
		setTimeout(function() {	resolve(delay);	}, delay);
	});
}; 

class ImageLoader {
	
	// Initially we once spawn a ffmpeg task which will connect to the camera.
	// ffmpeg will crop a certain image region and stream 32 bit RGBA tuplets.
	// We connect to the output pipe of the task and keep permanently
	// collecting chunks until we have a complete frame of ~ 100 kB.
	// When we receive a chunk of exactly 64kB we assume that this is only
	// some part of a frame. If the chunk is shorter we assume 
	// that it is the final chunk of a frame.
	// If there was no request to startRecognition() we will simply
	// throw away the completed frame and restart collecting chunks.
	// If there is a request to startRecognition() we will start detection
	// with the next frame. We convert the rgba32 pixels to (uncompressed)
	// tif format, load the tif data into the image-js library 
	// for further transformation and apply our detection logic. 
	// When detection is ready we transfer a compressed jpg image
	// to the web client)(s) and finally invoke a callback, passing the detection result.
	
	constructor(width,height,left,top) {
		// the crop area where we can find the combined symbols for rank and suit
		this.width	= width;
		this.height	= height;
		this.left	= left;
		this.top	= top;
		this.pixels = [];				// a binary matrix (height x width)
										// true = black pixel, false = white pixel
		this.blockFor(0);
		this.frameSize = this.width*this.height*4; // RGBA
		this.ffmpeg = null; // the streaming process
	}	
	
	log(msg) {
		console.log(Date.now()%100000+"\tImageLoader     "+msg);
	}
	
	startRecognition(whenRecognized) {
		// triggers a signal which will cause the frame collector to start detection
		// as soon as it has received the next complete frame
		// this function returns immediately to the caller
		// the detection will invoke the callback when completed
		this.whenRecognized = whenRecognized;
		this.triggered=true;
		this.log("recognition requested");
	}
	
	collectFrames(cmdLine) {
		// spawn the streaming process; it will do the cropping and it will
		// deliver frames at a decent rate of (e.g.) 5 frames per second
		this.log("starting ffmpeg stream "+cmdLine);
		this.ffmpeg = spawnProcess('ffmpeg', cmdLine.split(" "));
		
		var	rgbas = Buffer.alloc(0);	// buffer with pixel triplets

		var that = this;
		// action to be taken on reception of an RGBA chunk
		this.ffmpeg.stdout.on('data', async function (data) {
			// that.log("------------------------------------------------- "+String(data.length));
			try {
				// this.log("receiving data "+data.length);
				if(that.blockedUntil>Date.now()) {
					that.log("image loading blocked for "+String(that.blockedUntil-Date.now()));
					return;
				}
				else if (data.length==65536) {
					// if we got a full buffer we assume that this should be 
					// the first part of the (still incomplete) frame
					rgbas=Buffer.alloc(0);
					rgbas = Buffer.concat([rgbas,data]);
					// that.log("first data chunk received");
				}
				else if (data.length+rgbas.length!= that.frameSize) {
					// frane has wrong size, data possibly corrupted, ignore
					// that.log("corrupted frame, size is "+String(rgbas.length+data.length)+" instead of "+that.frameSize);
					return;
				}
				else if (!that.triggered) {
					// that.log("got frame, but no trigger");
					return;
				}
				else {
					that.log("got frame, starting detection");
					
					// there was a trigger and we just got the second chunk of a frame;
					// concatenate current data chunk to complete the frame
					// and create an uncompressed TIFF image					
					var tifImage = UTIF.encodeImage(Buffer.concat([rgbas,data]),that.width,that.height);
					// that.log("tif ready ",tifImage);

					// reset buffer and clear trigger, so we can continue catching frames
					rgbas=Buffer.alloc(0);
					that.triggered=false;

					// create image-js instance from TIFF image
					let image = await Image.load(tifImage);

					// perform detection; this will eventually invoke the callback
					// which was given when triggering the detection
					app.scanner.detectImage(image,that.whenRecognized);
				}
			} catch(e) {
				console.log(e);
			} 
		});

		this.ffmpeg.stderr.on('data', async function (data) {
			// sometimes we get messages about frames having been dropped
			// that.log("ffmpeg ERROR : "+data.toString());
		});

		this.ffmpeg.on('close', function (code) {
			that.log('ffmpeg process terminated, rc=' + code);
		});
		
		this.ffmpeg.on('exit', function (code) {
			that.log('ffmpeg process exited unexpectedly, code '+code);
		});
	}

	blockFor(msec) {
		this.log("blocking image loader for "+String(msec));
		this.blockedUntil = Date.now()+msec;
	}
	
	start(maxFrames) {
		// this.log("collecting "+maxFrames+" frames...");
		
		// let us collect maxFrames or forever if maxFrames<0
		// at a rate of 4 frames / second
		this.collectFrames(
			"-i /dev/video0"+(maxFrames>=0 ? " -vframes "+maxFrames : "")
			+ " -filter:v crop="+this.width+":"+this.height+":"+this.left+":"+this.top
			// +" -r 5 -f rawvideo -pix_fmt rgb32 -s "+this.width+"x"+this.height+" pipe:1"
			+" -f rawvideo -pix_fmt rgb32 -s "+this.width+"x"+this.height+" pipe:1"
		);
	}

}

class CardScanner {
	
	constructor(ranksFile) {
		if (fs.existsSync(ranksFile)) {
			this.rankFeatures=JSON.parse(fs.readFileSync(ranksFile));
			this.log("read rankFeatures, "+this.rankFeatures.length+" symbols");
			this.showFeatureMatrix(false);
		}
		else {
			this.log("no file with rankFeatures found");
			this.rankFeatures=[];
		}
	}
	
	log(msg) {
		Logger.log(Date.now()%100000+"\t  Scanner... "+msg);
	}
		
	setCropRegion(region) {
		this.cropArea = ""+region.width+":"+region.height+":"+region.x+":"+region.y;
	}

	scanOnly(whenScanned) {
		this.scan(false,"",0,whenScanned);
	}
	
	scanAndDeal(whenScanned) {
		this.scan(true,"",2,whenScanned);
	}
	
	async scan(shallDeal,shallLearnRank, retries, whenScanned) {
		// start scanning as soon as the current scan has terminated successfully

		this.shallDeal=shallDeal;
		this.shallLearnRank = shallLearnRank;

		// trigger the recognition - this will call detectImage()

		app.imageLoader.startRecognition(async result => {
			app.scanner.log("scan result is "+result);
			app.hardware.elms["camera"].dev.setSrc("data:image/jpeg;base64,"+this.imgTransformed.toBase64("image/jpeg"));
			if (!result && retries>0) {
				// before the last retry we retract the card a little
				if (retries==1) await app.engine.withDraw();
				await app.scan(shallDeal,shallLearnRank,retries-1,whenScanned);
			}
			else {
				if (typeof whenScanned != "undefined") whenScanned(result);
			}
		});	
	}
	
	detectImage(image,whenDetected) {
		// reduce color depth, rotate slightly, crop relevant area , flip sides
		// the resulting image will be transferred to the client(s) after detection
		this.imgTransformed = image
			.colorDepth(8)
			.rotate(2.5)
			.crop({x:10,y:10,width:90,height:270})
			.flipX()
		;
		
		// get red channel; the resulting image is the base for detection
		this.img = this.imgTransformed.getChannel(1);

		// the detect() function will invoke the callback when done
		this.detect(whenDetected);
	}
	
	async detect(whenDetected) {

		// calculate black threshold
		var black = this.img.getThreshold({algorithm:"otsu"});	
		this.log("scanning image "+this.img.width+" x "+this.img.height+"   ,   black= "+black);
		
		
		// get pixel array, convert to binary matrix using black threshold
		var pix = [];
		for (var i=0, y=-1; i < this.img.data.length; i++) {
			if (i % this.img.width === 0) pix[++y] = [];
			pix[y].push(this.img.data[i]<=black ? 1 : 0);
		}
		
		// remove some speckles which could jeopardize finding the correct image doimensions for rank and suit
		for (var x = 0; x < this.img.width-2;x++) {
			for (var y = 1; y<this.img.height-1; y++) {
				if ((pix[y][x] && !pix[y-1][x] && !pix[y+1][x] && (pix[y-1][x+1]+pix[y][x+1]+pix[y+1][x+1]) <= 2) 
				 || (pix[y][x] && (pix[y-1][x] + pix[y+1][x]<=1) && (pix[y-1][x+1]+pix[y][x+1]+pix[y+1][x+1]) <= 1)) {
					pix[y][x]=0;
					// this.log("despeckle "+x+" "+y);
				}
			}
		}
		
		var ok=true;
		
		// define an initial box which should cover some portion of rank AND suit
		var box = {  l: 35,  r: 60,  t:60,  b:200, 	// start box
					il:  0, ir:  0, it: 0, ib:  0,	// inner bound
					ol:  0, or:  0, ot: 0, ob:  0,	// outer bounding
					rl:  0, rr:  0, rt: 0, rb:  0, 	// rank
					sl:  0, sr:  0, st: 0, sb:  0 	// suit
		};

		// show matrix with start box
		// if (Logger.level>=1) this.showMatrix(pix,box);

		// this.log("start box: "+box.l+"..."+box.r+"   /   "+box.t+"..."+box.b);

		// shrink box until each border contains at least one black pixel

		ok = this.shrink(pix,box,"inner");
		if (!ok)	{
			this.log("NO inner box: "+box.l+"..."+box.r+"   /   "+box.t+"..."+box.b);
		}
		else {
			this.log("   inner box: "+box.l+"..."+box.r+"   /   "+box.t+"..."+box.b);
		}

		if (ok) {
			// enlarge box until each border does not contain any black pixel
			ok=this.enlarge(pix,box,"bound");
		}
		
		// show matrix with bounding box
		if (Logger.level>=1) this.showMatrix(pix,box);		
		
		if (!ok) {
			// tell result to client
			app.hardware.elms["card"].dev.println("?? ?");
			this.log("CARD NOT DETECTED");
			whenDetected(false);
			return;
		}
		
		var seq = 1; 	// allows despeckle; a value of 3 would demand 
						// a sequence of 3 black pixels to be considered as significant		

		// recognize card
		var rank = this.recognizeRank(box.rl,box.rr,box.rt,box.rb,black,seq,pix);
		// this.log("====> "+JSON.stringify(rank));
		var suit = this.recognizeSuit(box.sl,box.sr,box.st,box.sb,black,seq);
		// this.log("====> "+suit);
		
		// tell result to client
		app.card  = suit+rank.rank;
		app.owner = app.pbnDeal.getOwner(app.card);

		this.log("card ================================================================    "+app.owner+"   "+suit+" "+rank.rank+"  ("+rank.dif+")");

		var info = 	app.card+" "+app.owner + " (" + rank.type + " "+rank.dif+") -- " + 
					suit+rank.rank2+ " ("+rank.type2+" "+rank.dif2+")"
		;
		var info = 	app.card+(rank.type=="a"?"":rank.type)+" "+app.owner + "  ("+rank.dif+"/"+rank.dif2+")";

		app.hardware.elms["card"].dev.println(info);

		// suggest next rank to learn
		if (this.shallLearnRank!="" && this.shallLearnRank!="0") {
			var nextRank = app.nextRankSymbol(this.shallLearnRank);
			this.log("next rank to learn = "+nextRank);
			app.hardware.elms["learnRank"].dev.setValue(nextRank);
			theServer.broadcastState({id:"learnRank"},"TextInput",nextRank);
		}
		
		if (this.shallDeal) {
			this.cardNr++;
			await app.engine.ejectCard(app.card);
		}
		
		whenDetected(true);
	}

	recognizeRank(left,right,top,bottom,black,seq,pix) {
	
		// vertical white line ==> 10
		// if (this.whiteCol(left+Math.round((right-left)*0.2),right,top,bottom,1,black,seq,"10").col > 0) return "T";

		var features = this.calcFeatures(left,right,top,bottom,black,pix);
		
		if (this.shallLearnRank=="0") {
			this.rankFeatures=[];			
			this.log("all rank features cleared");
			return  {	rank : "0", 	type :"", 	dif : -1,
						rank2: "0", 	type2:"", 	dif2:-1
			};
		}

		if (this.shallLearnRank!="") {
			if (typeof this.rankFeatures == "undefined") this.rankFeatures = [];
			var found=false;
			for (var rf of this.rankFeatures) {
				if (rf.r+rf.t==this.shallLearnRank) {
					rf.f=features;
					found=true;
					break;
				}
			}
			if (!found) {
				this.rankFeatures.push({r:this.shallLearnRank.substring(0,1),t:this.shallLearnRank.substring(1),f:features});
			}
			// this.log(features);
			// show update feature matrix (per criterion)
			this.showFeatureMatrix(true);
		}
		
		var confidence = 700;
		var difs = this.rankDifs(features,-1);
		var match = this.bestMatch(features);
		if (match.length>=2 && match[0].dif < confidence) {
			this.log("recognize rank: best match ="+match[0].r+" ("+match[0].dif+")\t\t"+ match[1].r + " ("+match[1].dif+")");
			return  {	rank : match[0].r, 	type :match[0].t, 	dif :match[0].dif,
						rank2: match[1].r, 	type2:match[1].t, 	dif2:match[1].dif
			};			
		}
		else if (match.length>=1 && match[0].dif<confidence) {
			this.log("recognize rank: SINGLE MATCH");
			return  {	rank : match[0].r, 	type :match[0].t, 	dif :match[0].dif,
						rank2: "-", 		type2:"", 			dif2:-1
			};			
		}
		else {
			this.log("recognize rank: NO MATCH");
			console.log(features);
			
			if (match.length>0) {
				console.log(this.rankFeatures[match[0].r+match[0].t]);
				console.log(match);
				return {rank : "?",			type:"-",			dif:-1,
						rank2: match[0].r,	type2:match[0].t,	dif2:match[0].dif
				};
			}
			else {
				return {rank : "?",			type:"-",			dif:-1,
						rank2: "?",			type2:"-",			dif2:-1
				};
			}
				
		}
	}

	recognizeSuit(left,right,top,bottom,black,seq) {
		
		// top wider than center
		if ( this.rowFilled(left,right,top+(bottom-top+1)*.15, -1 ,1,black,"coeur >") >
			 this.rowFilled(left,right,top+(bottom-top+1)*.55, -1 ,1,black,"< couer")) return "c";

		// thin left
		if (!this.colFilled(top,bottom,left+(right-left+1)*.06,0.25,1,black,"not karo")) return "k";	// karo

		// top smaller than upper middle
		if ( this.rowFilled(left,right,top+(bottom-top+1)*.22, -1 ,1,black,"treff >") >
			 this.rowFilled(left,right,top+(bottom-top+1)*.33, -1 ,1,black,"< treff")) return "t";		// treff
		return "p";	// pik
	}
		
	shrink(pix,box,type) {
		box.il=box.l; box.ir=box.r; box.it=box.t; box.ib=box.b;
		var dir="l", shrunk=false;
		for (var n=1;n<200;n++) {
			if(dir=="l") {
				for(var y=box.it;y<=box.ib;y++) if (pix[y][box.il]) {dir="r"; break;}
				if(dir=="l") { shrunk=true; if(box.il>=this.img.width-1) break; box.il++; continue;}
			}
			if(dir=="r") {
				for(var y=box.it;y<=box.ib;y++) if (pix[y][box.ir]) {dir="t"; break;}
				if (dir=="r") { shrunk=true; if(box.ir<=0) break; box.ir--; continue;}
				dir="t";
			}
			if(dir=="t") {
				for(var x=box.il;x<=box.ir;x++) if (pix[box.it][x]) {dir="b"; break;}
				if (dir=="t") { shrunk=true; if(box.it>=this.img.height-1) break; box.it++; continue;}
				dir="b";
			}
			if(dir=="b") {
				for(var x=box.il;x<=box.ir;x++) if (pix[box.ib][x]) {dir="x"; break;}
				if (dir=="b") { shrunk=true; if(box.ib<=0) break; box.ib--; continue;}
				if (shrunk) { dir="l"; shrunk=false; continue; }
				break;
			}
		}

		if (shrunk)	{
			this.log("NO "+type+" box: "+box.il+"..."+box.ir+"   /   "+box.it+"..."+box.ib);
			return false;
		}
		else {
			// this.log("   "+type+" box: "+box.il+"..."+box.ir+"   /   "+box.it+"..."+box.ib);
			return true;
		}

	}
	
	enlarge(pix,box,type) {
		box.ol=box.il; box.or=box.ir; box.ot=box.it; box.ob=box.ib;
		
		var lw,rw,tw,bw;
		var dir="l"; 
		var enlarged=false;
		for (var n=1;n<250;n++) {
			if(dir=="l") {
				lw=true; for(y=box.ot;y<=box.ob;y++) if (pix[y][box.ol]) {lw=false; break;}
				if (!lw) {enlarged=true; if (box.ol<=0) break; box.ol--; continue;}
				dir="r";
			}
			if(dir=="r") {
				rw=true; for(y=box.ot;y<=box.ob;y++) if (pix[y][box.or]) {rw=false; break;}
				if (!rw) {enlarged=true; if(box.or>=this.img.width-1) break; box.or++; continue;}
				dir="t";
			}
			if(dir=="t") {
				tw=true; for(x=box.ol;x<=box.or;x++) if (pix[box.ot][x]) {tw=false; break;}
				if (!tw) {enlarged=true; if(box.ot<=0) break; box.ot--; continue;}
				dir="b";
			}
			if(dir=="b") {
				bw=true; for(x=box.ol;x<=box.or;x++) if (pix[box.ob][x]) {bw=false; break;}
				if (!bw) {enlarged=true; if(box.ob>=this.img.height-1) break; box.ob++; continue;}
				if (enlarged) { dir="l"; enlarged=false; continue; }
				break;
			}
		}
		box.rl=box.ol+1; box.rr=box.or-1; box.rt=box.ot+1;
		box.sl=box.ol+1; box.sr=box.or-1; box.sb=box.ob-1;
				
		// separate rank from suit symbol
		
		var allWhite;
		box.rb=box.st=box.rt;
		for(var y=box.rt;y<=box.sb;y++) {
			allWhite=true;
			for(var x=box.rl;x<=box.rr; x++) if (pix[y][x]) { allWhite=false; break; }
			if (allWhite) {box.rb=y-1; break; }
		}
		for(var y=box.rb+2;y<=box.sb;y++) {
			allWhite=true;
			for(var x=box.sl;x<=box.sr; x++) if (pix[y][x]) { allWhite=false; break; }
			if (!allWhite) {box.st=y; break; }
		}
		
		// this.log("   outer box rank LRTb "+box.rl+","+box.rr+","+box.rt);
		// this.log("   outer box suit LRtB "+box.sl+","+box.sr+","+box.sb);

		if (box.rb>=box.st) {
			this.log("no separation between rank and suit: "+box.rb+","+box.st);
			return false;			
		}
		// shrink rank and suit separately
		
		allWhite=true;
		for (var n=0;n<100;n++) {
			for(y=box.rt;y<=box.rb;y++) if (pix[y][box.rl]) {allWhite=false; break;}
			if (allWhite) box.rl++;
			else break;
		}
		allWhite=true;
		for (var n=0;n<100;n++) {
			for(y=box.st;y<=box.sb;y++) if (pix[y][box.sl]) {allWhite=false; break;}
			if (allWhite) box.sl++;
			else break;
		}
		allWhite=true;
		for (var n=0;n<100;n++) {
			for(y=box.rt;y<=box.rb;y++) if (pix[y][box.rr]) {allWhite=false; break;}
			if (allWhite) box.rr--;
			else break;
		}
		allWhite=true;
		for (var n=0;n<100;n++) {
			for(y=box.st;y<=box.sb;y++) if (pix[y][box.sr]) {allWhite=false; break;}
			if (allWhite) box.sr--;
			else break;
		}

		// this.log("   outer box rank LRTB "+box.rl+","+box.rr+","+box.rt+","+box.rb);
		// this.log("   outer box suit LRTB "+box.sl+","+box.sr+","+box.st+","+box.sb);

		if (box.rl<0 || box.rr<0 || box.sl<0 || box.sr<0 || box.rb-70<box.rt || box.sb-40<box.st || box.sb>=285) {
			this.log("enlarging box failed: "+box.rl+","+box.rr+","+box.sl+","+box.sr);
			return false;			
		}

		if (box.rt==box.st || box.rb==box.sb) {
			this.log("no separation between rank and suit: "+box.rt+"..."+box.sb);
			return false;			
		}

		if (enlarged) {
			this.log("no bound box: "+box.rl+"..."+box.rr+" / "+box.rt+"..."+box.rb+ " ---- "
						+box.sl+"..."+box.sr+" / "+box.st+"..."+box.sb);
			return false;
		}
		else {
			// this.log("   bound box: "+box.rl+"..."+box.rr+" / "+box.rt+"..."+box.rb+ " ---- "
			//			+box.sl+"..."+box.sr+" / "+box.st+"..."+box.sb);
			return true;
		}
	}

	showMatrix(data,box) {
		var line="";
		for(var x=0;x<data[0].length;x++) {
			if (x%10==8) line+=(""+(x+2)).padStart(3);
			else if (x%10==0) line+="       ";
		}
		console.log("    "+line);
		for (var y=0; y<data.length; y++) {
			if (y<box.ot || y>box.ob) continue;
			var line=(""+y).padStart(3)+" ";
			for (var x=0; x<data[0].length; x++) {
				if 		((x==box.rl || x==box.rr) && (y>=box.rt&&y<=box.rb) ) line+= (data[y][x]?"R":"|");
				else if ((x==box.sl || x==box.sr) && (y>=box.st&&y<=box.sb) ) line+= (data[y][x]?"S":"|");
				else if ((x==box.ol || x==box.or) && (y>=box.ot&&y<=box.ob) ) line+= (data[y][x]?"O":" ");
				else if ((x==box.il || x==box.ir) && (y>=box.it&&y<=box.ib) ) line+= (data[y][x]?"I":"i");
				else if ((y==box.rt || y==box.rb) && (x> box.rl&&x< box.rr) ) line+= (data[y][x]?"R":"-");
				else if ((y==box.st || y==box.sb) && (x> box.sl&&x< box.sr) ) line+= (data[y][x]?"S":"-");
				else if ((y==box.ot || y==box.ob) && (x> box.ol&&x< box.or) ) line+= (data[y][x]?"O":" ");
				else if ((y==box.it || y==box.ib) && (x> box.il&&x< box.ir) ) line+= (data[y][x]?"I":"i");
				else line=line+(data[y][x]?"x":".");
			}
			console.log(line);
		}
		line="";
		for(var x=0;x<data[0].length;x++) {
			if (x%10==8) line+=(""+(x+2)).padStart(3);
			else if (x%10==0) line+="       ";
		}
		console.log("    "+line);
	}
	
	blackCol(fromX,toX,fromY,toY,grid,black,seq,id) {
		// find the left-most column with at least seq joining black pixels
		var data, b=0;
		for (var col = fromX;col<toX;col+=grid) {
			data = this.img.getColumn(col,0);
			for (var row=fromY; row<toY; row+=grid) {
				if (data[row]<black) { 
					if(++b>=seq) {
						this.log(""+id+"\tblack column ("+b+") found at "+col+"   (row="+(row+1-b)+")" );
						return { col:col,row:row+1-b };
					}
				}
				else b=0;
			}
		}
		this.log(""+id+"\tno black col ("+seq+") found ("+fromX+"/"+toX+" .. "+fromY+"/"+toY);
		return { col:-1,row:-1 };
	}

	whiteCol(fromX,toX,fromY,toY,grid,black,seq,id) {
		// find the left most column without a sequence of seq black pixels
		var found,data,b=0;
		for (var col=fromX;col<toX;col+=grid) {
			data = this.img.getColumn(col,0);
			found=false;
			for (var row=fromY; row<toY; row+=grid) {
				if (data[row]<black) { if(++b>=seq) { found=true; break; } }
				else b=0;
			}
			if (!found) {
				this.log(""+id+"\twhite column found at "+col+"   (row="+(row+1-b)+")");
				return { col:col,row:row+1-b };
			}
		}
		this.log(""+id+"\tno white col ("+seq+") found ("+fromX+"/"+toX+" .. "+fromY+"/"+toY);
		return { col:-1,row:-1 };
	}

	whiteRow(fromX,toX,fromY,toY,grid,black,seq,id) {
		// find the top-most row without a contingent number of seq black pixels
		var found,data,b=0;
		for (var row = fromY;row<toY;row+=grid) {
			if (row<0) break;
			data = this.img.getRow(row,0);
			found=false;
			for (var col=fromX; col<toX; col+=grid) {
				if (data[col]<black) { if (++b>=seq) {found=true; break;} }
				else b=0;
			}
			if (!found) {
				this.log(""+id+"\twhite row found at "+row+"   (col="+(col+1-b)+")");
				return { col:col+1-b,row:row };
			}
		}
		this.log(""+id+"\tno white row ("+seq+") found ("+fromX+"/"+toX+" .. "+fromY+"/"+toY);
		return { col:-1,row:-1 };
	}

	blackRow(fromX,toX,fromY,toY,grid,black,seq,id) {
		// find the top-most row with at least seq joining black pixels
		var data,b=0;
		for (var row = fromY;row<toY;row+=grid) {
			data = this.img.getRow(row,0);
			for (var col=fromX; col<toX; col+=grid) {
				if (col<0) break;
				if (data[col]<black) {
					if (++b>=seq) {
						this.log(""+id+"\tblack row ("+b+") found at "+row+"   (col="+(col+1-b)+")");
						return { col:col+1-b,row:row };
					}
				}
				else b=0;
			}
		}
		this.log(""+id+"\tno black row ("+seq+") found ("+fromX+"/"+toX+" .. "+fromY+"/"+toY);
		return { col:-1,row:-1 };
	}

	rowFilled(fromX,toX,row,val,grid,black,id) {
		// return true/false if share of black pixels in row exceeds the value
		row=Math.round(row);
		var data = this.img.getRow(row,0);
		var n=0;
		for (var col=fromX; col<=toX; col+=grid) {
			if (data[col]<black) n++;
		}
		var result = val<0 ? n*grid/(toX-fromX+1) : n*grid/(toX-fromX+1) >= val;
		// this.log(""+id+"\t"+(result?"YES":"NO")+"\t"+Math.round(100*n*grid/(toX-fromX+1)) + "% / "+(val*100)+"%  filled	in row "+row);
		return result;
	}

	colFilled(fromY,toY,col,val,grid,black,id) {
		// return true/false if share of black pixels in col exceeds the value
		col=Math.round(col);
		var data = this.img.getColumn(col,0);
		var n=0;
		for (var row=fromY; row<=toY; row+=grid) {
			if (data[row]<black) n++;
		}
		var result = val<0 ? n*grid/(toY-fromY+1) : n*grid/(toY-fromY+1) >= val;
		// this.log(""+id+"\t"+(result?"YES":"NO")+"\t"+Math.round(100*n*grid/(toY-fromY+1)) + "% / "+(val*100)+"%  filled	in col "+col);
		return result;
	}
	
	freeWay(col,row,fromCol,toCol,fromRow,toRow,pix) {
		// normalized sequence of non-black fields in W N E S direction
		var freeSteps=[];
		var c, r;
		for (c=col; c>=fromCol && !pix[row][c];c--) ; freeSteps[0]= Math.round(1000 * (col-c) / (toCol-fromCol+1));
		for (c=col; c<=  toCol && !pix[row][c];c++) ; freeSteps[1]= Math.round(1000 * (c-col) / (toCol-fromCol+1));
		for (r=row; r>=fromRow && !pix[r][col];r--) ; freeSteps[2]= Math.round(1000 * (row-r) / (toRow-fromRow+1));
		for (r=row; r<=  toRow && !pix[r][col];r++) ; freeSteps[3]= Math.round(1000 * (r-row) / (toRow-fromRow+1));
		return freeSteps;
	}

	freeWayX(col,row,fromCol,toCol,fromRow,toRow,pix) {
		// normalized sequence of non-black fields in NW NE SE SW direction
		var freeSteps=[];
		var c, r;
		// console.log("testing "+col+","+row);
		for (c=col,r=row; c>=fromCol && r<=  toRow && !pix[r][c];c--,r++) ; freeSteps[0]= Math.round(1000 * (col-c) / (toCol-fromCol+1));
		for (c=col,r=row; c<=  toCol && r<=  toRow && !pix[r][c];c++,r++) ; freeSteps[1]= Math.round(1000 * (c-col) / (toCol-fromCol+1));
		for (c=col,r=row; c<=  toCol && r>=fromRow && !pix[r][c];c++,r--) ; freeSteps[2]= Math.round(1000 * (c-col) / (toCol-fromCol+1));
		for (c=col,r=row; c>=fromCol && r>=fromRow && !pix[r][c];c--,r--) ; freeSteps[3]= Math.round(1000 * (col-c) / (toCol-fromCol+1));
		return freeSteps;
	}

	horizontalSlot(fromCol,toCol,fromRow,toRow,pix) {
		// look for horizontal white lines ("slots") with >=75% width
		// starting at the left/right border in the upper lower region
				
		var freeSteps=[];
		var c, r;
		var rows = toRow-fromRow+1;
		var ru1=Math.round(fromRow+0.15*rows);
		var ru2=Math.round(fromRow+0.45*rows);
		var rl1=Math.round(fromRow+0.55*rows);
		var rl2=Math.round(fromRow+0.85*rows);
		var max,n;
		// upper left
		for (max=0, r=ru1;r<=ru2;r++) {
			for (n=0, c=fromCol; c<=toCol; c++,n++) if (pix[r][c]) break;
			if (n>max) {
				// console.log("ul "+r+" "+n);
				max=n;
			}
		}
		freeSteps[0]= Math.round(1000 * max / (toCol-fromCol+1));
		// upper right
		for (max=0, r=ru1;r<=ru2;r++) {
			for (n=0, c=toCol; c>=fromCol; c--,n++) if (pix[r][c]) break;
			if (n>max) {
				// console.log("ur "+r+" "+n);
				max=n;
			}
		}
		freeSteps[1]= Math.round(1000 * max / (toCol-fromCol+1));
		// lower left
		for (max=0, r=rl1;r<=rl2;r++) {
			for (n=0, c=fromCol; c<=toCol; c++,n++) if (pix[r][c]) break;
			if (n>max) {
				// console.log("ll "+r+" "+n);
				max=n;
			}
		}
		freeSteps[2]= Math.round(1000 * max / (toCol-fromCol+1));
		// lower right
		for (max=0, r=rl1;r<=rl2;r++) {
			for (n=0, c=toCol; c>=fromCol; c--,n++) if (pix[r][c]) break;
			if (n>max) {
				// console.log("lr "+r+" "+n);
				max=n;
			}
		}
		freeSteps[3]= Math.round(1000 * max / (toCol-fromCol+1));

		return freeSteps;
	}
	
	verticalSlot(fromCol,toCol,fromRow,toRow,pix) {
		// look for vertical white lines ("slots") with >=75% width
		// starting at the top/bottom border in the left/right region
				
		var freeSteps=[];
		var c, r;
		var cols = toCol-fromCol+1;
		var cl1=Math.round(fromCol+0.10*cols);
		var cl2=Math.round(fromCol+0.45*cols);
		var cr1=Math.round(fromCol+0.55*cols);
		var cr2=Math.round(fromCol+0.90*cols);
		var max,n;
		// top left
		for (max=0, c=cl1;c<=cl2;c++) {
			for (n=0, r=fromRow; r<=toRow; r++,n++) if (pix[r][c]) break;
			if (n>max) {
				// console.log("tl "+c+" "+n);
				max=n;
			}
		}
		freeSteps[0]= Math.round(1000 * max / (toRow-fromRow+1));
		// bottom left
		for (max=0, c=cl1;c<=cl2;c++) {
			for (n=0, r=toRow; r>=fromRow; r--,n++) if (pix[r][c]) break;
			if (n>max) {
				// console.log("bl "+c+" "+n);
				max=n;
			}
		}
		freeSteps[1]= Math.round(1000 * max / (toRow-fromRow+1));
		// top right
		for (max=0, c=cr1;c<=cr2;c++) {
			for (n=0, r=fromRow; r<=toRow; r++,n++) if (pix[r][c]) break;
			if (n>max) {
				// console.log("tr "+c+" "+n);
				max=n;
			}
		}
		freeSteps[2]= Math.round(1000 * max / (toRow-fromRow+1));
		// bottom right
		for (max=0, c=cr1;c<=cr2;c++) {
			for (n=0, r=toRow; r>=fromRow; r--,n++) if (pix[r][c]) break;
			if (n>max) {
				// console.log("br "+c+" "+n);
				max=n;
			}
		}
		freeSteps[3]= Math.round(1000 * max / (toRow-fromRow+1));

		return freeSteps;
	}
		
	calcFeatures(left,right,top,bottom,black,pix) {		
		var cr 	= Math.round(left+(right-left+1)*0.60);
		var mid	= Math.round((top+bottom)*0.48);
		var up1 = Math.round((top+(bottom-top+1)*0.200));
		var up3 = Math.round((top+(bottom-top+1)*0.300));
		var up5 = Math.round((top+(bottom-top+1)*0.800));
		var features = [];
		features.push(this.freeWay (left+4,	top+4,		left,right,top,bottom,pix));
		features.push(this.freeWay (right-4,top+4,		left,right,top,bottom,pix));
		features.push(this.freeWay (left+4,	bottom-4,	left,right,top,bottom,pix));
		features.push(this.freeWay (right-4,bottom-4,	left,right,top,bottom,pix));
		features.push(this.freeWay (left+4,	mid,		left,right,top,bottom,pix));
		features.push(this.freeWay (right-4,mid,		left,right,top,bottom,pix));
		features.push(this.freeWayX(cr,		up1,		left,right,top,bottom,pix));
		features.push(this.freeWayX(cr,		up3,		left,right,top,bottom,pix));
		features.push(this.freeWayX(cr,		up5,		left,right,top,bottom,pix));
		features.push(this.freeWayX(cr,		mid,		left,right,top,bottom,pix));
		features.push(this.horizontalSlot(left,right,top,bottom,pix));
		features.push(this.verticalSlot  (left,right,top,bottom,pix));
		
		// console.log(left,cr,right,"  //  ",top,up1,up3,mid,up5,up7,bottom);
		// console.log(features);

		return features;		
	}

	featuresDif(f1,f2,crit) {
		// sum of squared differences
		// if crit < 0 : all features, else only the critical feature
		var dif=0;
		for (var f=0;f<Math.min(f1.length,f2.length);f++) {
			if (crit>=0 && f!=crit) continue;
			for (var d=0;d<3;d++) {
				dif += (f1[f][d]-f2[f][d]) * (f1[f][d]-f2[f][d]) ;
			}
		}
		return Math.round(dif/100./ (crit>=0 ? 1 : Math.min(f1.length,f2.length)));
	}

	rankDifs(features,crit) {
		var difs=[];
		for (var rf of this.rankFeatures) {
			difs.push({r:rf.r,t:rf.t,dif:this.featuresDif(features,rf.f,crit)});	
		}
		return difs;
	}

	bestMatch(features) {
		var difs = this.rankDifs(features,-1);
		difs.sort(function(a,b){ return a.dif-b.dif; });
		return difs;		
	}
	
	showFeatureMatrix(details) {
		
		if (typeof this.rankFeatures=="undefined") return;
		// console.log(this.rankFeatures);
		console.log("==============================================================================================");
		var header="";
		var numFeatures=0;
		for (var rf of this.rankFeatures) {
			if (numFeatures==0) numFeatures=rf.f.length;
			header+="\t"+rf.r+" "+rf.t;
		}
		for (var crit = -1; crit<numFeatures; crit++) {			
			console.log("----------------------------------------------------------------------------------------------");
			console.log(header+" ::::: "+crit);
			var r=0,rr;
			for (var rf of this.rankFeatures) {
				var line=rf.r+" "+rf.t+"\t";			
				var difs = this.rankDifs(rf.f,crit);
				rr=0;
				for(var dif of difs) {
					if (rr>r)				line+=".\t";
					else if (r==rr)			line+="*\t";
					else if (dif.dif < 50)	line+=".\t";
					else					line+=Math.round(dif.dif/100.)+"\t";
					rr++;
				}
				console.log(line);
				r++;
			}
			console.log(header);
			if (!details) break;
		}
		console.log("==============================================================================================");
	}
	
	print(left,right,top,bottom,black,compressX,compressY,repeatHeader,spotX,spotY) {
		if(left<0) left=0;
		if(right>this.img.width) right=this.img.width;
		if(top<0) top=0;
		if(bottom>this.img.height) bottom=this.img.height;

		var measure="  ";
		for(var x=left;x<right;x++) {
			if 		(x%10>0 && x%10<=6) measure+=" ";
			else if (x-left<6) measure+" ";
			else if (x%10==7) measure += (""+(x+3)).padStart(4);
		} 
		var line,data;
		this.log("----------------------------------------------------------------------- "+left+" .. "+right+"   //   "+top+" .. "+bottom);
		for (var row = top; row<bottom;row+=compressY) {
			if (row<10) line="   "+row;
			else if (row<100) line= "  "+row;
			else if (row<1000) line= " "+row;
			else line=""+row;
			if ((row==top) || (row+top)%repeatHeader==0) this.log((""+(row+top)).padStart(4)+measure);
			data = this.img.getRow(row,0);
			for (var col=left; col<right; col+=compressX) {
				if (row==spotY && col==spotX) line += data[col]>black ? "o" : "O";
				else line += data[col]>black ? " " : "#";
			}
			line+="|";
			this.log(line);
		}	
		this.log("----------------------------------------------------------------------- END");
	}
	
}

class EngineInterface {
	
	constructor() {
		this.engine	= null;
		this.cardNr	= 0;
	}
	
	log(msg) {
		Logger.log(Date.now()%100000+"\t  Engine.... "+msg);
	}
	
	start(port,baudRate) {
		try {
			const SerialPort = require("serialport");
			const Readline = require("@serialport/parser-readline");
			const parser = new Readline();
			
			var that=this;
			this.engine = new SerialPort(port, { baudRate: baudRate });	
			this.engine.pipe(parser);
			this.engine.on("open", () => {
				parser.on("data", data => {
					that.onRead(data.toString().trim());
				});
				return true;
			});
		}
		catch(err) {
			return false;
		}
	}
	
	onRead(data) {
		this.log(data);
	}
	
	async zeroPos() {
		this.engine.write("z");
		this.bin="N";
		await resolveAfter(2500);
	}
	
	async unloadPos() {
		this.engine.write("u");
		await resolveAfter(1000);
	}
	
	async withDraw() {
		this.engine.write("w");
		await resolveAfter(150);
	}
	
	ejectCard(card) {
		var bin=app.pbnDeal.getOwner(card);
		this.log("Engine       card "+card+" belongs to "+bin);
		this.ejectBin(bin);
	}

	ejectBin(bin) {
		if (bin==".") bin=this.bin;
		// calculate travelTime for rotation of card catching bins
		var travelTime=250; // 90 degrees
		if (bin==this.bin) travelTime=0;	// same position
		else if ((bin=="N" && this.bin=="S") || (bin=="E" && this.bin=="W")) travelTime=400; // opposite position
		this.bin=bin;
		this.log("Engine       ejecting to bin "+bin);
		this.engine.write(""+bin);
		this.cardNr++;
		// block the imageLoader until the bottom card has been ejected 
		// and until the card above has been retracted to a position 
		// where it can be reliably scanned
		app.imageLoader.blockFor(700+travelTime);
	}
}	

class PbnLib {
	
	constructor(file) {
		var text = fs.readFileSync(file).toString('utf-8');
		this.deals=[];
		this.dealIds = [];
		var skip=true;
		var nr=0;
		for(var sect of text.split(/\r?\n\[Board /i)) {
			if (skip) { skip=false; continue; }
			var deal = new PbnDeal("\n[Board "+sect);
			this.deals.push(deal);
			nr++;
			this.dealIds.push(nr+" ["+
				((deal.board==nr) ? "" : (deal.board+" "))+deal.dealer+" "+
				deal.vulnerable+"] "+
				deal.contract+" von "+deal.declarer+" "+deal.result)
			;
		}
	}
}

class PbnDeal {
	
	constructor(text) {

		this.owner={};
		var nextPlayers = {N:"E",E:"S",S:"W",W:"N"};
		this.auction="";
		this.tricks="";
		var state="";
		for(var line of text.split(/\r?\n/)) {
			if (line.substring(0,1)=="[") state="";
			if (line.trim()=="" || line.substring(0,1)=="%") continue;

			if (line.substr(0,7).toUpperCase()=='[BOARD ') {
				this.auction="";
				this.tricks="";
				var state="";
				this.board=line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,'');
				// console.log("NEW PbnDeal: BOARD "+line+" :::: "+this.board);
			}
			else if (line.substr(0,8).toUpperCase()=='[DEALID ') {
				this.dealID=line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,'');
			}
			else if (line.substr(0,8).toUpperCase()=='[DEALER ') {
				this.dealer=line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,'');
			}
			else if (line.substr(0,12).toUpperCase()=='[VULNERABLE ') {
				this.vulnerable=line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,'').replace("None","--").replace("EW","OW").replace("All","alle");
			}
			else if (line.substr(0,9).toUpperCase()=='[AUCTION ') {
				this.auction = "<br/>Gebote   "+line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,'')+"  =============<br/>";
				state="auctionStart";
			}
			else if (line.substr(0,10).toUpperCase()=='[CONTRACT ') {
				this.contract=line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,'')
					.replace(/C/g,"♣").replace(/D/g,"♦").replace(/H/g,"♥").replace(/S/g,"♠").replace(/NT/g,"SA").trim();
			}
			else if (line.substr(0,10).toUpperCase()=='[DECLARER ') {
				this.declarer=line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,'').replace("E","O");
			}
			else if (line.substr(0,6).toUpperCase()=='[PLAY ') {
				this.tricks = "Stiche   "+line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,'')+"  ==========<br>";
				state="tricksStart";
			}
			else if (line.substr(0,7).toUpperCase()=='[SCORE ') {
				this.score = line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,'');
			}
			else if (line.substr(0,8).toUpperCase()=='[RESULT ') {
				this.tricks=parseInt(line.replace(/^.[A-Za-z]+ +"/,'').replace(/".*/,''));
				this.result=(this.tricks-6)-parseInt(this.contract);
				if (this.result==0) this.result="=";
				else if (this.result>0) this.result="+"+this.result;
				else this.result = ""+this.result;
			}
			else if (line.substr(0,6).toUpperCase()=='[DEAL ') {
				// line=line.replace(/Q/g,'D').replace(/J/g,'B').replace(/T/g,'Z');
				var player = line[7];
				var hands=line.replace(/^.[A-Za-z]+ +"/,'').split(/[ "\]]/,4);
				var once=true;
				for(var hand of hands) {
					var suits=hand.split(".");
					if (once) { suits[0]=suits[0].substr(2); once=false; }
					for(var c=0;c<suits[0].length;c++) this.owner["p"+suits[0][c]]=player;
					for(var c=0;c<suits[1].length;c++) this.owner["c"+suits[1][c]]=player;
					for(var c=0;c<suits[2].length;c++) this.owner["k"+suits[2][c]]=player;
					for(var c=0;c<suits[3].length;c++) this.owner["t"+suits[3][c]]=player;
					player=nextPlayers[player];
				}
				// this.log(this.owner);
			}
			if (state=="auction") {
				this.auction+=line+"<br/>";
			} 
			if (state=="auctionStart") state="auction";
			if (state=="tricks") {
				this.tricks+=line+"<br/>";
			} 
			if (state=="tricksStart") state="tricks";
		}
		if (false && text.indexOf("{")>0) {				
			this.layout=text
				.replace(/.*?\{(.*?)\}.*/s,"$1")
				.replace(/C/g,"♣")
				.replace(/D/g,"♦")
				.replace(/H/g,"♥")
				.replace(/S/g,"♠")
				.replace(/\n/sg,"\n  ")
			;
		}
		else {
			this.layout= "Board "+ (this.dealID || this.board);
			var played = this.contract + " "+this.declarer+" "+this.result+ "  "+this.score;
			this.layout += "                                                  "
							.substring(0,49-played.length-this.layout.length)+played;
			this.layout+="<br/>";
			var sym={p:"♠",c:"♥",k:"♦",t:"♣"};
			for (var s of ["p","c","k","t"]) {
				this.layout+="                 "+sym[s];
				for (var r of ["A","K","Q","J","T","9","8","7","6","5","4","3","2"]) {
					if (this.owner[s+r]=="N") this.layout+=" "+r;
				}
				this.layout+="<br>";
			}
			for (var s of ["p","c","k","t"]) {
				var n=0;
				this.layout+= " "+sym[s];
				for (var r of ["A","K","Q","J","T","9","8","7","6","5","4","3","2"]) {
					if (this.owner[s+r]=="W") {
						n++;
						this.layout+=" "+r;
					}
				}
				this.layout+="                                ".substring(0,31-2*n)+sym[s];
				for (var r of ["A","K","Q","J","T","9","8","7","6","5","4","3","2"]) {
					if (this.owner[s+r]=="E") {
						this.layout+=" "+r;
					}
				}
				this.layout+="<br>";
			}
			for (var s of ["p","c","k","t"]) {
				this.layout+="                 "+sym[s];
				for (var r of ["A","K","Q","J","T","9","8","7","6","5","4","3","2"]) {
					if (this.owner[s+r]=="S") this.layout+=" "+r;
				}
				this.layout+="<br>";
			}

			this.layout+= "Teiler "+ this.dealer;
			this.layout+= "                                                  "
							.substring(0,34-this.vulnerable.length)+"Gefahr "+this.vulnerable;
			this.layout+="<br/>";
			
			this.layout+=this.auction+"<br/>"+this.tricks;
		}		
	}
	
	getOwner(card) {
		return this.owner[card] || "?";
	}
}