"use strict";

var _common = require("test/common");

var _channel = require("../channel");

describe('we check the validateChannel method', function () {
  it('check for a RGB image', function () {
    let image = new _common.Image(2, 2, {
      kind: 'RGB'
    });
    expect((0, _channel.validateChannel)(image, 'r')).toBe(0);
    expect((0, _channel.validateChannel)(image, 'g')).toBe(1);
    expect((0, _channel.validateChannel)(image, 'b')).toBe(2);
    expect(function () {
      (0, _channel.validateChannel)(image, 'a');
    }).toThrow(/does not contain alpha/);
    expect(function () {
      (0, _channel.validateChannel)(image);
    }).toThrow(/the channel has to be/);
    expect(function () {
      (0, _channel.validateChannel)(image, 3, false);
    }).toThrow(/the channel has to be/);
  });
  it('check for a RGBA image', function () {
    let image = new _common.Image(2, 2, {
      kind: 'RGBA'
    });
    expect((0, _channel.validateChannel)(image, 'r')).toBe(0);
    expect((0, _channel.validateChannel)(image, 'g')).toBe(1);
    expect((0, _channel.validateChannel)(image, 'b')).toBe(2);
    expect((0, _channel.validateChannel)(image, 'a')).toBe(3);
    expect(function () {
      (0, _channel.validateChannel)(image);
    }).toThrow(/the channel has to be/);
    expect(function () {
      (0, _channel.validateChannel)(image, 3, false);
    }).toThrow(/alpha channel may not be/);
  });
  it('check for a GreyA image', function () {
    let image = new _common.Image(2, 2, {
      kind: 'GREYA'
    });
    expect((0, _channel.validateChannel)(image, 0)).toBe(0);
    expect((0, _channel.validateChannel)(image, 1)).toBe(1);
    expect((0, _channel.validateChannel)(image, 'a')).toBe(1);
    expect((0, _channel.validateChannel)(image, 'a', true)).toBe(1);
    expect(function () {
      (0, _channel.validateChannel)(image, 'r');
    }).toThrow(/undefined channel/);
    expect(function () {
      (0, _channel.validateChannel)(image, 1, false);
    }).toThrow(/alpha channel may not/);
    expect(function () {
      (0, _channel.validateChannel)(image, 'a', false);
    }).toThrow(/alpha channel may not/);
  });
});